#include <LiquidCrystal_I2C.h> // Library for LCD Display
#include <ArduinoIoTCloud.h>   // Arduino IoT Cloud library
#include <Arduino_ConnectionHandler.h> // For WiFi connection


// Define pins
#define SOIL_MOISTURE_SENSOR_PIN A0
#define RAIN_SENSOR_PIN A1
#define RELAY_PIN 7
#define PUMP_PIN 8


// LCD Display setup (I2C address, columns, rows)
LiquidCrystal_I2C lcd(0x27, 16, 2);


// Arduino IoT Cloud variables
bool systemOn; // ON/OFF switch
int soilMoisture; // Soil moisture level (0-100%)
bool isRaining; // Rain detection status


// WiFi credentials
const char SSID[] = "YOUR_WIFI_SSID";
const char PASS[] = "YOUR_WIFI_PASSWORD";


// Thresholds
const int DRY_THRESHOLD = 20; // 20% soil moisture
const int WET_THRESHOLD = 85; // 85% soil moisture
const int RAIN_THRESHOLD = 500; // Rain sensor threshold


void setup() {
  // Initialize serial communication
  Serial.begin(9600);


  // Initialize LCD
  lcd.begin();
  lcd.backlight();
  lcd.print("Initializing...");


  // Initialize pins
  pinMode(SOIL_MOISTURE_SENSOR_PIN, INPUT);
  pinMode(RAIN_SENSOR_PIN, INPUT);
  pinMode(RELAY_PIN, OUTPUT);
  pinMode(PUMP_PIN, OUTPUT);
  digitalWrite(RELAY_PIN, LOW); // Ensure relay is off initially
  digitalWrite(PUMP_PIN, LOW); // Ensure pump is off initially


  // Connect to Arduino IoT Cloud
  initIoTCloud();
}


void loop() {
  // Update Arduino IoT Cloud
  ArduinoIoTCloud.update();


  // Read sensor values
  int soilMoistureValue = analogRead(SOIL_MOISTURE_SENSOR_PIN);
  int rainSensorValue = analogRead(RAIN_SENSOR_PIN);


  // Convert soil moisture value to percentage (0-100%)
  soilMoisture = map(soilMoistureValue, 0, 1023, 0, 100);


  // Check if it's raining
  isRaining = (rainSensorValue < RAIN_THRESHOLD);


  // Display soil moisture and rain status on LCD
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Moisture: ");
  lcd.print(soilMoisture);
  lcd.print("%");


  if (isRaining) {
    lcd.setCursor(0, 1);
    lcd.print("IT'S RAINING");
  }


  // Automated watering logic
  if (systemOn) {
    if (soilMoisture <= DRY_THRESHOLD && !isRaining) {
      startWatering();
    } else if (soilMoisture >= WET_THRESHOLD || isRaining) {
      stopWatering();
    }
  } else {
    stopWatering();
  }


  delay(1000); // Delay for stability
}


void startWatering() {
  digitalWrite(RELAY_PIN, HIGH); // Turn on relay
  digitalWrite(PUMP_PIN, HIGH); // Turn on pump
  lcd.setCursor(0, 1);
  lcd.print("Watering...");
}


void stopWatering() {
  digitalWrite(RELAY_PIN, LOW); // Turn off relay
  digitalWrite(PUMP_PIN, LOW); // Turn off pump
}


void initIoTCloud() {
  // Connect to WiFi
  WiFi.begin(SSID, PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to WiFi...");
  }
  Serial.println("Connected to WiFi");


  // Initialize Arduino IoT Cloud
  ArduinoIoTCloud.begin();
  ArduinoIoTCloud.addProperty(systemOn, READWRITE, ON_CHANGE, NULL);
  ArduinoIoTCloud.addProperty(soilMoisture, READ, ON_CHANGE, NULL);
  ArduinoIoTCloud.addProperty(isRaining, READ, ON_CHANGE, NULL);
}


void onSystemOnChange() {
  // Handle ON/OFF switch change
  if (systemOn) {
    lcd.setCursor(0, 1);
    lcd.print("System ON");
  } else {
    lcd.setCursor(0, 1);
    lcd.print("System OFF");
    stopWatering();
  }
}